@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: Apache-2.0
+// SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 /* Copyright 2013-2019 IBM Corp. */
 
 #include <skiboot.h>
@@ -296,31 +296,10 @@ static void show_all_regs(struct npu2 *npu, int brick_index)
 	}
 }
 
-void npu2_dump_scoms(int chip_id)
+void npu2_dump_scoms(struct npu2 *npu, int chip_id)
 {
-	struct npu2 *npu;
-	struct phb *phb;
-	struct npu2_dev *dev;
-
-	/*
-	 * Look for the npu2 structure for that chip ID. We can access it
-	 * through the array of phbs, looking for a nvlink or opencapi
-	 * phb. We can have several entries, but they all point
-	 * to the same npu2 structure
-	 */
-	for_each_phb(phb) {
-		npu = NULL;
-		if (phb->phb_type == phb_type_npu_v2) {
-			npu = phb_to_npu2_nvlink(phb);
-		} else if (phb->phb_type == phb_type_npu_v2_opencapi) {
-			dev = phb_to_npu2_dev_ocapi(phb);
-			npu = dev->npu;
-		}
-		if (npu && npu->chip_id == chip_id) {
-			show_all_regs(npu, -1 /* all bricks */);
-			break;
-		}
-	}
+	if (npu && npu->chip_id == chip_id)
+		show_all_regs(npu, -1 /* all bricks */);
 }
 
 static uint64_t npu2_ipi_attributes(struct irq_source *is __unused, uint32_t isn __unused)
@@ -406,6 +385,13 @@ static void npu2_err_interrupt(struct irq_source *is, uint32_t isn)
 			p->chip_id, irq_name);
 		free(irq_name);
 		show_all_regs(p, brick);
+		/*
+		 * P9 NPU doesn't support recovering a link going down
+		 * unexpectedly. So we mark the device as broken and
+		 * report it to the OS, so that the error is logged
+		 * and the drivers notified.
+		 */
+		npu2_opencapi_set_broken(p, brick);
 		opal_update_pending_evt(OPAL_EVENT_PCI_ERROR,
 					OPAL_EVENT_PCI_ERROR);
 		break;
@@ -530,7 +516,6 @@ static struct npu2 *setup_npu(struct dt_node *dn)
 	npu->index = dt_prop_get_u32(dn, "ibm,npu-index");
 	npu->chip_id = gcid;
 	npu->xscom_base = dt_get_address(dn, 0, NULL);
-	npu->phb_index = dt_prop_get_u32(dn, "ibm,phb-index");
 
 	init_lock(&npu->i2c_lock);
 	npu->i2c_pin_mode = ~0; // input mode by default
